import { DEFAULT_MAX_CONCURRENT_REQUESTS, StatusEnum } from "./constants.js";
class DataLoader {
  static maxConcurrent = DEFAULT_MAX_CONCURRENT_REQUESTS;
  static started = 0;
  static cachedData = {};
  static queue = {};
  key;
  method;
  isCalled = false;
  isCancelled = true;
  status = StatusEnum.IDLE;
  error;
  data;
  observers = [];
  timeout;
  loadCount = 0;
  isFirstLoading = true;
  dataUpdatedAt;
  constructor(args) {
    this.key = args.key;
    this.method = args.method;
    if (args.enabled) {
      this.status = StatusEnum.LOADING;
    }
    this.data = DataLoader.cachedData[this.key];
    if (args.notifyChanges) this.observers.push(args.notifyChanges);
  }
  notifyChanges() {
    if (this.timeout) {
      clearTimeout(this.timeout);
    }
    this.timeout = setTimeout(() => {
      this.observers.forEach((observer) => observer());
    });
  }
  getData() {
    return DataLoader.cachedData[this.key];
  }
  tryLaunch = async () => {
    if (DataLoader.started < DataLoader.maxConcurrent) {
      DataLoader.started += 1;
      DataLoader.queue[this.key] = this.launch();
    } else {
      DataLoader.queue[this.key] = new Promise((resolve) => {
        setTimeout(resolve);
      }).then(this.tryLaunch);
    }
    return DataLoader.queue[this.key];
  };
  load = async (force = false) => {
    if (force || !this.isCalled) {
      this.isCalled = true;
      if (this.status !== StatusEnum.LOADING) {
        this.status = StatusEnum.LOADING;
        this.notifyChanges();
      }
      DataLoader.queue[this.key] = this.tryLaunch();
    }
    return DataLoader.queue[this.key];
  };
  launch = async () => {
    try {
      this.isCancelled = false;
      this.loadCount += 1;
      const data = await this.method();
      if (!this.isCancelled) {
        DataLoader.cachedData[this.key] = data;
        this.status = StatusEnum.SUCCESS;
        this.data = data;
        this.error = void 0;
        this.dataUpdatedAt = Date.now();
      }
      this.isCalled = false;
      this.isFirstLoading = false;
      DataLoader.started -= 1;
      delete DataLoader.queue[this.key];
      this.notifyChanges();
      return data;
    } catch (error) {
      if (!this.isCancelled) {
        this.status = StatusEnum.ERROR;
        this.error = error;
      }
      this.isCalled = false;
      this.isFirstLoading = false;
      DataLoader.started -= 1;
      delete DataLoader.queue[this.key];
      this.notifyChanges();
      if (!this.isCancelled) {
        throw error;
      }
      return void 0;
    }
  };
  clearData() {
    DataLoader.cachedData[this.key] = void 0;
  }
  cancel() {
    DataLoader.started -= 1;
    delete DataLoader.queue[this.key];
    this.isCancelled = true;
    this.status = StatusEnum.IDLE;
  }
  addObserver(observer) {
    this.observers.push(observer);
  }
  removeObserver(observer) {
    const index = this.observers.indexOf(observer);
    if (index > -1) this.observers.splice(index, 1);
  }
}
export {
  DataLoader as default
};
