import { useRef, useState, useCallback, useMemo, useEffect } from "react";
import { useDataLoaderContext } from "./DataLoaderProvider.js";
import { StatusEnum } from "./constants.js";
import { marshalQueryKey } from "./helpers.js";
const useInfiniteDataLoader = (baseKey, method, baseParams, pageParamKey, config) => {
  const {
    getOrAddRequest,
    computeKey,
    onError: onGlobalError
  } = useDataLoaderContext();
  const {
    enabled = true,
    onError,
    onSuccess,
    keepPreviousData = false,
    initialData,
    dataLifetime,
    getNextPage
  } = config ?? {};
  const requestRefs = useRef([]);
  const [page, setPage] = useState(baseParams[pageParamKey]);
  const nextPageRef = useRef(page);
  const getNextPageRef = useRef(getNextPage);
  const methodRef = useRef(
    () => method(
      pageParamKey && page ? { ...baseParams, [pageParamKey]: page } : baseParams
    )
  );
  const paramsRef = useRef(
    pageParamKey && page ? { ...baseParams, [pageParamKey]: page } : baseParams
  );
  const onSuccessRef = useRef(onSuccess);
  const onErrorRef = useRef(onError ?? onGlobalError);
  const [, setCounter] = useState(0);
  const forceRerender = useCallback(() => {
    setCounter((current) => current + 1);
  }, []);
  const baseQueryKey = useMemo(() => marshalQueryKey(baseKey), [baseKey]);
  useEffect(
    () => () => {
      requestRefs.current.forEach(
        (request2) => request2.removeObserver(forceRerender)
      );
    },
    [forceRerender]
  );
  const getCurrentRequest = () => {
    const currentQueryKey = marshalQueryKey([
      baseQueryKey,
      "infinite",
      page
    ]);
    requestRefs.current.forEach(
      (request2) => !request2.key.startsWith(computeKey(baseQueryKey)) ? request2.removeObserver(forceRerender) : void 0
    );
    requestRefs.current = requestRefs.current.filter(
      ({ key }) => key.startsWith(computeKey(baseQueryKey))
    );
    const requestInRef = requestRefs.current.find(
      (request2) => request2.key.endsWith(currentQueryKey)
    );
    if (!requestInRef) {
      const request2 = getOrAddRequest(currentQueryKey, {
        enabled,
        method: methodRef.current
      });
      requestRefs.current.push(request2);
      request2.addObserver(forceRerender);
      return request2;
    }
    return requestInRef;
  };
  const request = getCurrentRequest();
  const needLoad = useMemo(
    () => !!(enabled && (!request.dataUpdatedAt || !dataLifetime || request.dataUpdatedAt && dataLifetime && request.dataUpdatedAt + dataLifetime < Date.now())),
    [enabled, request.dataUpdatedAt, dataLifetime]
  );
  const optimisticIsLoadingRef = useRef(needLoad);
  const previousDataRef = useRef(request.data);
  const isLoading = requestRefs.current.some(
    (req) => req.status === StatusEnum.LOADING || optimisticIsLoadingRef.current
  );
  const isLoadingFirstPage = (request.status === StatusEnum.LOADING || optimisticIsLoadingRef.current) && requestRefs.current.length <= 1;
  const isSuccess = request.status === StatusEnum.SUCCESS;
  const isError = request.status === StatusEnum.ERROR;
  const isIdle = requestRefs.current.every(
    (req) => req.status === StatusEnum.IDLE && !enabled
  );
  const reload = useCallback(async () => {
    await Promise.all(
      requestRefs.current.map(
        (req) => req.load(true).then(onSuccessRef.current).catch(onErrorRef.current)
      )
    );
  }, []);
  useEffect(() => {
    request.method = () => method(paramsRef.current);
  }, [method, request]);
  useEffect(() => {
    onSuccessRef.current = onSuccess;
  }, [onSuccess]);
  useEffect(() => {
    onErrorRef.current = onError ?? onGlobalError;
  }, [onError, onGlobalError]);
  useEffect(() => {
    if (keepPreviousData) {
      previousDataRef.current = request.data;
    }
  }, [request.data, keepPreviousData]);
  useEffect(() => {
    getNextPageRef.current = getNextPage;
  }, [getNextPage]);
  useEffect(() => {
    paramsRef.current = pageParamKey && page ? { ...baseParams, [pageParamKey]: page } : baseParams;
  }, [baseParams, pageParamKey, page]);
  useEffect(() => {
    if (needLoad) {
      const defaultOnSuccessOrError = () => {
      };
      const onSuccessLoad = onSuccessRef.current ?? defaultOnSuccessOrError;
      const onFailedLoad = onErrorRef.current ?? defaultOnSuccessOrError;
      request.load().then(async (result) => {
        if (getNextPageRef.current) {
          nextPageRef.current = getNextPageRef.current(
            result,
            paramsRef.current
          );
        }
        await onSuccessLoad(result);
      }).catch(onFailedLoad);
    }
    optimisticIsLoadingRef.current = false;
  }, [needLoad, request]);
  const loadMore = useCallback(() => {
    setPage(nextPageRef.current);
  }, []);
  const data = useMemo(
    () => ({
      isIdle,
      isError,
      isLoading,
      isSuccess,
      hasNextPage: nextPageRef.current !== void 0,
      isLoadingFirstPage,
      data: isLoadingFirstPage || [...requestRefs.current].filter((dataloader) => !!dataloader.data).length === 0 ? initialData : [...requestRefs.current].filter((dataloader) => !!dataloader.data).map((dataloader) => dataloader.data),
      error: request.error,
      reload,
      loadMore
    }),
    [
      initialData,
      isIdle,
      isLoading,
      isSuccess,
      isError,
      request.error,
      isLoadingFirstPage,
      reload,
      loadMore
    ]
  );
  return data;
};
export {
  useInfiniteDataLoader
};
